# 1 "fac2cr.F"
# 1 "<command-line>"
# 1 "fac2cr.F"
c *********************************************************************
c     f90 -g -o fac2cr fac2cr.F dlapack.o dblas.o dslatec.o (-save-temps)
c     f90 -g -o fac2cr fac2cr.F -llapack -lblas -lslatec
c *********************************************************************

      module cdat ! useful data & character constants

      real*8, parameter :: zero = 0.d0
      real*8, parameter :: one  = 1.d0
      real*8, parameter :: two  = 2.d0

      real*8, parameter :: alpha = 1.d0/137.035 989 5 d0
      real*8, parameter :: pi = 3.141 592 653 589 793 d0
      real*8, parameter :: Ry = 13.605 698 1 d0             !eV
      real*8, parameter :: a0 = 0.529 177 249 d-8           !cm
      real*8, parameter :: hc = 12398.419                   !eV A

      character*2  els(0:99)
      character*12 elname(0:99)

      end module

c *********************************************************************
c *********************************************************************

      program fac2cr

! Generates input files for Cretin from FAC output

! Usage -   fac2cr  [isomin [isomax]]  [nophxs]  [addcx]
!         Run this in a directory containing those FAC output files
!         (and only those FAC files) which should be combined into an
!         atomic model.  Other files in the directory will be ignored.

!         If isomin [isomax] are specified, only files for sequences
!         within this limits will be included.  This has no effect if
!         all sequences are in a single file.

!         If nophxs is specified, photoexcitations will appear in a
!         "rad_rate" data section rather than a "phxs" section.

!         If addxs is specified, collisional excitation data will be
!         added even if no collisional excitation files are present.
!         Otherwise, collisional excitation data will only be added
!         to augment data contained in collisional excitation files.

!         NOTE: structure files may only have one NLEV section per
!               sequence, although filelist can now interpret
!               multiple sections per sequence.


      use cdat
      implicit none

      integer niso, nfit, nsh, msh
      parameter (niso = 100, nfit = 64, nsh = 56, msh = 21)

      real*8, allocatable  :: E(:,:), Ez(:), E2(:)
      real*8, allocatable  :: tj(:,:), tj2(:)
      real*8, allocatable  :: x(:), sig(:), sigf(:), cs(:)
      real*8, allocatable  :: xsig(:), esig(:)
      integer, allocatable :: indst(:,:), indx(:,:), i0(:)
      integer, allocatable :: nliso(:), npq(:,:)
      integer, allocatable ::  nps(:,:,:), nps1(:), nps2(:)
      integer, allocatable :: done(:,:)
      character*24, allocatable :: cname(:,:),cjj(:,:)
      character*24, allocatable :: cname2(:),cjj2(:)

      real*8  rjnk,Ein,tjin,gf,Ar,Aw,dE
      real*8  c0,c3,Ef,csin,gul,gll,Ew,Efau,Ewau
      real*8  sig0,emax,rnorm,rmax,rmax2,rmaxc,ematch
      real*8  cf1,cf2,cf3,cf4,xx,yy,px,rfac,shalf,fout,xPmax
      integer iso,isoi,isoi2,isof,isomin,i,ii,i2,imax,isoin,
     &        j,jj,jj2,jnk,
     &        ul,ul0,ll,gu,gl,ntr,nP,nP0,nP1,nPmax,k,kk,lin,ic,ic0,
     &        multip,multmax,
     &        nlev,nlev0,ntot,nlist,nstr,nferr3,nlq,
     &        ierr,ierrc,iferr0,iferr1,iferr2,iferr3,iferr4,
     &        inone,istype,idx,idy,ifiso1,ifiso2
      integer shpop(nsh),npop(nsh)

      integer nGmax, ng

      logical       uta, phxs, addcx

      character*1   chr
      character*3   el
      character*13  ofile,efile,pfile
      character*16  name
      character*21  cnamein,cin,cjjin
      character*32  cshin
      character*32  filename,arg
      character*32  flist(niso,0:11)
      character*144 str

      integer       iargc, isitr, isxtr

c     data xPmax  / 1.d6 /       ! max x-value for collisional fits
      data xPmax  / 1.d2 /       ! max x-value for collisional fits

      data ematch / 5.d-3 /
      data rmax, rmax2, rmaxc / 2.d0, 10.d0, 10.d0 /

      data shpop / 2, 2,6, 2,6,10, 2,6,10,14, 2,6,10,14,18,
     &             2,6,10,14,18,22,  2,6,10,14,18,22,26,
     &             2,6,10,14,18,22,26,30,  2,6,10,14,18,22,26,30,34,
     &             2,6,10,14,18,22,26,30,34,38, 0 /

c ---------------------------------------------------------------------

! the current coding is partially independent of formatting, but depends
! on identifying the starting location of the ascii names as 2 spaces
! before the first '*' and does a formatted read from that location

  801 format (a21)
  802 format (a21)
  803 format (a21)

c ---------------------------------------------------------------------

  890 format (/1x,'Writing atomic model ',a)
  900 format (/,'data',4x,a)
  905 format (/,'c ls_shells')
  910 format (/,'end data')
  920 format (/,2x,'enot',2x,i3,3x,a8,3x,f11.6)
  922 format (/,2x,'enot',2x,i3,3x,a8,3x,f11.5)
  924 format (/,2x,'enot',2x,i3,3x,a8,3x,f11.4)
  926 format (/,2x,'enot',2x,i3,3x,a8,3x,f11.3)
  930 format (4x,'elev',2x,i3,1x,i5,3x,a16,f11.0,1x,f10.5,2x,26i3,i4)
  940 format (4x,'esublev',2x,i3,1x,i4,1x,i4,2x,a10,f10.0,1x,es10.3)
  950 format (2x,'d',2x,i3,1x,i5,2x,i3,1x,i5,2x,1p8e13.5)
  951 format ( 'c d',2x,i3,1x,i5,2x,i3,1x,i5,2x,1p8e13.5)
  955 format (2x,'d',2x,i3,1x,i5,1x,i4,2x,i3,1x,i5,1x,i4,2x,1p8e13.5)
  960 format (2x,'d',2x,i3,3x,1p2e13.5)
  970 format (4x,a4,2x,i3,1x,i4,2x,1p9e13.5)
  980 format ('per',3x,i3,3x,i3)

  985 format (/,'c ... ',a,/)
  990 format (/,'c ',78('.'))
  995 format ()

   99 format (/,' ####  unrecognized argument: ',a)

c ---------------------------------------------------------------------

      uta   = .false.
      phxs  = .true.
      addcx = .false.

      nlist = 0
      flist = ' '
      call filelist(flist,nlist,el,niso,nlev,imax)

      nstr = len(str)

      lin = 2
      if (el(2:2).eq.' ') lin = 1
      el(3:3) = '_'

      isoi = 0
      isof = 0
      do iso = 1,niso
         if (flist(iso,1).ne.' ') then
            if (isoi.eq.0) isoi = iso
            isof = iso
         endif
      enddo
      isomin = isoi

      nlist = iargc()

      ierr   = 0
      ifiso1 = 0
      ifiso2 = 0
      do i = 1,nlist
         call getarg(i,arg)
         if (arg.eq.'nophxs') then
            phxs = .false.
         elseif (arg.eq.'addcx') then
            addcx = .true.
         elseif (ifiso1.eq.0) then
            read (arg,*) iso
            isoi = max(isoi,iso)
            isomin = isoi
            ifiso1 = 1
         elseif (ifiso2.eq.0) then
            read (arg,*) iso
            isof = min(isof,iso)
            ifiso2 = 1
         else
            write (*,99) arg
            ierr = 1
         endif
      enddo

      if (ierr.ne.0) stop

      ofile = el(1:lin)//'_atom.dat'   ! output file name
      efile = el(1:lin)//'_atom.err'   ! error file name
      pfile = el(1:lin)//'_atom.plt'   ! plot file name

      nPmax = 6                ! maximum number of input points for cross sections
      nGmax = 40               ! maximum number of groups for evaluating cross sections

      iferr0 = 0               ! flag for match warnings
      iferr1 = 0               ! flag for match warnings
      iferr2 = 0               ! flag for fitting errors
      iferr3 = 0               ! flag for bad fits

      call setdat

      nPmax = max(nPmax,nfit)

      imax = imax + 1

      allocate (
     &   E(isoi-1:isof,imax), Ez(isoi-1:isof), E2(imax),
     &   tj(isoi-1:isof,imax), tj2(imax),
     &   x(nPmax), sig(nPmax), sigf(nPmax), cs(nPmax),
     &   xsig(0:nGmax), esig(0:nGmax) )

      E    = zero ; Ez  = zero ; E2   = zero
      tj   = zero ; tj2 = zero
      x    = zero ; sig = zero ; sigf = zero ; cs = zero
      xsig = zero ; esig = zero

      allocate (
     &   indst(isoi-1:isof,imax),
     &   indx(imax,2),
     &   i0(isoi-1:isof),
     &   nliso(isoi-1:isof),
     &   npq(isoi-1:isof,imax),
     &   nps(isoi-1:isof,imax,nsh),
     &   nps1(nsh), nps2(nsh) )

      indst = 0 ; indx = 0 ; i0  = 0 
      nliso = 0 ; npq  = 0 ; nps = 0 

      allocate (
     &   cname(isoi-1:isof,imax), cname2(imax),
     &   cjj(isoi-1:isof,imax), cjj2(imax) )

      write (*,890) ofile
      write (*,995)

      open  (2,file=ofile)
      open  (3,file=efile)
      open  (4,file=pfile)

c ---------------------------------------------------------------------
c     read & record level structure
c ---------------------------------------------------------------------

! read in level structure from structure files

      do iso = isoi,isof
         filename = flist(iso,1)
         if (filename.eq.' ') cycle

         write (*,*) '  structure file  ',filename
         open (1,file=filename)

         do i = 1,8
           read (1,*, err=200,iostat=ierr) cin
         enddo

         read (1,*, err=200,iostat=ierr) cin,cin, isoin
         read (1,*, err=200,iostat=ierr) cin,cin, nlev
         read (1,*, err=200,iostat=ierr) cin

 100     nliso(iso) = nliso(iso) + nlev

         if (isoin.ne.iso) then
            stop
         endif

         do i = 1,nlev
            read (1,'(a)') str
            read (str,*, err=200,iostat=ierr) j,jnk,Ein,jnk,nlq,tjin

            cshin = ' '
            cin   = ' '
            cjjin = ' '

            idx = index(str,'*') - 2
            do while ((str(idx:idx).eq.' ') .and. (idx.lt.nstr))
               idx = idx + 1
            enddo
            idy = idx + 20
            do while (index(str(idy:nstr),'*').ne.0)
               idy = idy + index(str(idy:nstr),'*') + 2
            enddo
            read (str(idx:idy),801, err=200,iostat=ierr) cshin

            idx = idy + 1
            do while ((str(idx:idx).eq.' ') .and. (idx.lt.nstr))
               idx = idx + 1
            enddo
            read (str(idx:nstr),802, err=200,iostat=ierr) cin

            idx = idx + 21
            do while ((str(idx:idx).eq.' ') .and. (idx.lt.nstr))
               idx = idx + 1
            enddo
            read (str(idx:nstr),803, err=200,iostat=ierr) cjjin

            tjin = tjin + 1

            E(iso,j+1)     = Ein
            tj(iso,j+1)    = tjin
            npq(iso,j+1)   = abs(nlq/100)
            cname(iso,j+1) = cin
            cjj(iso,j+1)   = cjjin

            nps(iso,j+1,:) = 0

! assign LS shell populations by parsing cshin and cname
            ic0  = 1
            ii   = 0
            npop = 0
            do ic = 1,32
               chr = cshin(ic:ic)
               if (chr.eq.'*') then
                  read (cshin(ic0:ic-1),*, err=200,iostat=ierr) ii
                  ic0 = ic
                  cycle
               elseif (chr.eq.' ') then
                  if (ii.eq.0) then
                     ic0 = ic
                     cycle
                  endif
                  read (cshin(ic0+1:ic-1),*, err=200,iostat=ierr)
     &                 npop(ii)
               else
                  cycle
               endif

               if (npop(ii).eq.2*ii**2) then
                  do jj = (ii-1)*ii/2+1,ii*(ii+1)/2
                     nps(iso,j+1,jj) = shpop(jj)
                  enddo
               endif

               ii  = 0
               ic0 = ic
            enddo

            cshin = cname(iso,i)

            ii  = 0
            kk  = 0
            nlq = -1
            do ic = 1,21
               chr = cshin(ic:ic)
               if (cshin(ic:ic).eq.' ') then
                  if ((ii.eq.0).or.(nlq.lt.0)) then
                     ii = 0
                     cycle
                  endif
                  read (cshin(ic0+1:ic-1),*) k
                  jj = (ii-1)*ii/2 + nlq + 1
                  if (jj.lt.nsh) then
                     nps(iso,j+1,jj) = k
                     if (k.lt.kk) then
                        kk = kk-k
                        do i2 = 0,nlq-1
                           jj2 = (ii-1)*ii/2 + i2 + 1
                           nps(iso,j+1,jj2) = min(kk,2*i2+1)
                           kk = kk - nps(iso,j+1,jj2)
                        enddo
                     endif
                  else
                     nps(iso,j+1,nsh) = jj
                  endif
                  ii  = 0
                  kk  = 0
                  nlq = -1
               elseif (chr.eq.'*') then
                  nlq = 0
                  ic0 = ic
               elseif (chr.eq.'s') then
                  nlq = 0
                  ic0 = ic
               elseif (chr.eq.'p') then
                  nlq = 1
                  ic0 = ic
               elseif (chr.eq.'d') then
                  nlq = 2
                  ic0 = ic
               elseif (chr.eq.'f') then
                  nlq = 3
                  ic0 = ic
               else
                  jj = ichar(chr)
                  if ((jj.ge.70).and.(jj.le.90)) then
                     nlq = jj - 67
                     ic0 = ic
                  elseif ((jj.ge.102).and.(jj.le.122)) then
                     nlq = jj - 99
                     ic0 = ic
                     if (jj.ge.107) then   ! skip 'j'
                        nlq = nlq-1
                     endif
                  elseif ((jj.ge.48).and.(jj.le.57)) then
                     if (nlq.ge.0) then
                        kk = 10*kk + (jj-48)
                     else
                        ii = 10*ii + (jj-48)
                     endif
                  else
                     nlq = -1
                     ic0 = ic
                  endif
               endif
            enddo

c ... fill in shells identified in npop but not in cname
c ... assume these are the lowest shells for this N
            do ii = 1,6
               ntot = 0
               do jj = (ii-1)*ii/2+1,ii*(ii+1)/2
                  ntot = ntot + nps(iso,j+1,jj)
               enddo
               ntot = npop(ii) - ntot
               if (ntot.gt.0) then
                  do jj = (ii-1)*ii/2+1,ii*(ii+1)/2
                     if (nps(iso,j+1,jj).eq.0) then
                        nps(iso,j+1,jj) = min(ntot,shpop(jj))
                        ntot = max(ntot-shpop(jj),0)
                     endif
                  enddo
               endif
            enddo

c ... take care of full shells not already accounted for
c ... assume these are the lowest shells
            ntot = iso
            do ii = 1,nsh-1
               ntot = ntot - nps(iso,j+1,ii)
            enddo
            if (nps(iso,j+1,nsh).ne.0) ntot = ntot - 1

            do ii = 1,nsh-1
               if (nps(iso,j+1,ii).eq.0) then
                  if (ntot.ge.shpop(ii)) then
                     nps(iso,j+1,ii) = shpop(ii)
                     ntot = ntot - shpop(ii)
                  endif
               endif
            enddo
         enddo

! read in level structure for previous isosequence (used for indexing ionizations)
! check match with data from previous structure file

         indx = 0
         iferr0 = 0
         iferr1 = 0

         read (1,*, end=200,iostat=ierr) cin,cin, isoin
         read (1,*, err=200,iostat=ierr) cin,cin, nlev0
         read (1,*, err=200,iostat=ierr) cin

         if (isoin.eq.iso) then
            nlev = nlev0
            goto 100
         endif

         if (isoin.eq.0) then
            isomin = min(isomin,isoin)
         endif

         if (isoin.lt.isomin) cycle

         do i = 1,nlev0
            read (1,'(a)', err=200,iostat=ierr) str
            read (str,*, err=200,iostat=ierr) j,jnk,Ein,jnk,nlq,tjin

            cshin = ' '
            cin   = ' '
            cjjin = ' '

            idx = index(str,'*') - 2
            do while ((str(idx:idx).eq.' ') .and. (idx.lt.nstr))
               idx = idx + 1
            enddo
            read (str(idx:nstr),801, err=200,iostat=ierr) cshin

            idx = idx + 21
            do while ((str(idx:idx).eq.' ') .and. (idx.lt.nstr))
               idx = idx + 1
            enddo
            read (str(idx:nstr),802, err=200,iostat=ierr) cin

            idx = idx + 21
            do while ((str(idx:idx).eq.' ') .and. (idx.lt.nstr))
               idx = idx + 1
            enddo
            read (str(idx:nstr),803, err=200,iostat=ierr) cjjin

            tjin = tjin + 1

            if (isoin.eq.iso-1 .and. i0(isoin).eq.0) then
               i0(isoin) = j       ! offset to first level of ionized ion in transition files
               Ez(iso)   = Ein     ! set ionization potential
               jj = 1
               indst(isoin,jj) = 1 ! matching index
               indx(1,1) = 1
               indx(1,2) = 1
            endif

! equivalent index from previous structure file
            jj = j-i0(isoin)+1

            E2(jj)     = Ein - Ez(iso)
            tj2(jj)    = tjin
            cname2(jj) = cin
            cjj2(jj)   = cjjin

! assign energies of yet-undetailed ion
            if (isoin.eq.isoi-1) then
               E(isoin,jj)     = Ein-Ez(iso)
               tj(isoin,jj)    = tjin
               cjj(isoin,jj)   = cjjin
               cname(isoin,jj) = cin
               indst(isoin,jj) = jj

! check to see whether levels in ionized ion match previous data
            elseif (isoin.eq.iso-1) then
               ii  = 2
               Ein = Ein - Ez(iso)

! try to match name, configuration, 2J
               do while (indst(isoin,jj).eq.0 .and. ii.le.nlev0)
                  if (cname(isoin,ii).eq.cin  .and.
     &                cjj(isoin,ii).eq.cjjin  .and.
     &                tj(isoin,ii).eq.tjin    .and.
     &                indx(ii,1).eq.0         ) then
                     indst(isoin,jj) = ii
                     indx(ii,1) = 1
                     indx(jj,2) = 1
                  endif
                  ii = ii+1
               enddo

! try to match configuration, 2J, energy
               if (indst(isoin,jj).eq.0) then
                  ii = 2
                  do while (indst(isoin,jj).eq.0 .and. ii.le.nlev0)
                     Ef = abs(Ein-E(isoin,ii))/Ein
                     if (cname(isoin,ii).eq.cin  .and.
     &                   tj(isoin,ii).eq.tjin    .and.
     &                   Ef.lt.ematch            .and.
     &                   indx(ii,1).eq.0         ) then
                        indst(isoin,jj) = ii
                        indx(ii,1) = 1
                        indx(jj,2) = 1
                     endif
                     ii = ii+1
                  enddo
               endif

! try to match 2J, energy
               if (indst(isoin,jj).eq.0) then
                  ii = 2
                  do while (indst(isoin,jj).eq.0 .and. ii.le.nlev0)
                     Ef = abs(Ein-E(isoin,ii))/Ein
                     if (tj(isoin,ii).eq.tjin    .and.
     &                   Ef.lt.ematch            .and.
     &                   indx(ii,1).eq.0         ) then
                        indst(isoin,jj) = ii
                        indx(ii,1) = 1
                        indx(jj,2) = 1
                     endif
                     ii = ii+1
                  enddo
               endif

! if no match found: go ahead with default assignment
               if (indst(isoin,jj).eq.0) then
                  indst(isoin,jj) = jj
               endif

! test for mismatched names
               ii = indst(isoin,jj)
               if (indx(ii,1).ne.0 .and. indx(jj,2).ne.0) then
                  if (cjj(isoin,ii).ne.cjjin) then
                     if (iferr1.eq.0) then
                        write (*,*) 'warning: mismatched level names',
     &                              ' - see ',efile
                        write (3,*)
                        write (3,*) 'mismatched level names for ',
     &                              els(isoin)//'-like'
                        iferr1 = 1
                     endif
                     write (3,*) '    ------'
                     write (3,'(3x,a6,i6,f10.2,f6.0,3x,2a23)')
     &                  'file 1:  ',ii,E(isoin,ii),
     &                  tj(isoin,ii),cname(isoin,ii),cjj(isoin,ii)
                     write (3,'(3x,a6,i6,f10.2,f6.0,3x,2a23)')
     &                  'file 2: ',jj,Ein,tjin,cin,cjjin
                  endif
               endif
            endif

         enddo

! test for unmatched levels
         do i = 1,nlev0
            ii = indx(i,1)
            if (ii.eq.0) then
               if (iferr0.eq.0) then
                  write (*,*) 'warning: unmatched levels - see ',
     &                         efile
                  write (3,*)
                  write (3,*) 'unmatched levels for ',
     &                        els(isoin)//'-like'
                  write (3,*) '    ------'
                  iferr0 = 1
               endif
               write (3,'(3x,a6,i6,f10.2,f6.0,3x,2a23)')
     &            'file 1: ',i,E(isoin,i),tj(isoin,i),
     &            cname(isoin,i),cjj(isoin,i)
            endif
         enddo
         if (iferr0.ne.0) then
            write (3,*) '    ------'
         endif
         do i = 1,nlev0
            ii = indx(i,2)
            if (ii.eq.0) then
               write (3,'(3x,a6,i6,f10.2,f6.0,3x,2a23)')
     &            'file 2: ',i,E2(i),tj2(i),
     &            cname2(i),cjj2(i)
            endif
         enddo

 200     close (1)

         if (ierr.gt.0) then
            write (*,*) 'Error # ',ierr,' reading file ',filename
         elseif (ierr.lt.0) then
c           write (*,*) 'Unexpected EOF or EOR in file ',filename
         endif

      enddo

      isoi2 = max(isoi,isomin+1)

! record level structure
      write (2,900) 'model'
      write (2,905)

      do iso = isomin,isof
         name = els(iso) // '-like'
         if (Ez(iso).lt.1.d1) then
            write (2,920) iso,name(1:8),Ez(iso)
         elseif (Ez(iso).lt.1.d2) then
            write (2,922) iso,name(1:8),Ez(iso)
         elseif (Ez(iso).lt.1.d3) then
            write (2,924) iso,name(1:8),Ez(iso)
         else
            write (2,926) iso,name(1:8),Ez(iso)
         endif
         write (2,995)

         i = 1
         do while (i.eq.1 .or. E(iso,i).gt.zero)

! replace ' ' with '_'
            do jj = 1,23
               if (cname(iso,i)(jj:jj).eq.' ') cname(iso,i)(jj:jj)='_'
               if (cjj(iso,i)(jj:jj).eq.' ') cjj(iso,i)(jj:jj)='_'
            enddo

            write (cnamein,'(a2,a2,a2,i4)')
     &         cname(iso,i)(1:2),cname(iso,i)(5:6),
     &         cname(iso,1)(9:10),int(tj(iso,i))

            if (cnamein(7:7).eq.' ') cnamein(7:7) = '_'
            if (cnamein(8:8).eq.' ') cnamein(8:8) = '_'
            if (cnamein(9:9).eq.' ') cnamein(9:9) = '_'

!           cnamein = cname(iso,i)(1:2)//cname(iso,i)(5:6)//
!    &                cname(iso,i)(9:10)//setch(int(tj(iso,i)))

            str = ' '
            write (str(1:19),930) iso,i
            write (str(22:31),'(a)') cnamein(1:10)

            if (tj(iso,i).lt.1.d7) then
               write (str(33:42),'(f10.0)') tj(iso,i)
            else
               write (str(33:42),'(1pe10.4)') tj(iso,i)
            endif

            if (E(iso,i).lt.1.d1) then
               write (str(45:56),'(f11.8)') E(iso,i)
            elseif (E(iso,i).lt.1.d2) then
               write (str(45:56),'(f11.7)') E(iso,i)
            elseif (E(iso,i).lt.1.d3) then
               write (str(45:56),'(f11.6)') E(iso,i)
            else
               write (str(45:56),'(f11.5)') E(iso,i)
            endif

            ii = 0
            do jj = msh+1,nsh
               if (nps(iso,i,jj).gt.0) ii = jj
            enddo

            write (str(59:129),'(22i3,i4)')
     &            (nps(iso,i,j),j=1,msh),ii,npq(iso,i)

            write (2,'(a)') str(1:129)
            i = i+1
         enddo
      enddo

      write (2,910)

c ---------------------------------------------------------------------
c     read & record radiative decay
c ---------------------------------------------------------------------

      write (2,990)
      if (phxs) then
         write (2,900) 'phxs'
      else
         write (2,900) 'rad_rate'
      endif

c ... Fac_1_1_1 gives strange answers for quadrupole transitions (and
c ... others!) for detailed (non-UTA) calculations.  Setting multmax=1
c ... will ignore quadrupole (and higher) transitions for detailed models.

      multmax = 2

      ii = 0
      jj = 0
      do iso = isoi,isof
         filename = flist(iso,2)
         if (filename.eq.' ') cycle

         nlev = nliso(iso)
         allocate (done(nlev,nlev))
         done = 0

         write (2,985) els(iso)//'-like'

         write (*,*) '  datafile  ',filename
         open (1,file=filename)

         do i = 1,7
            read (1,*, err=300,iostat=ierr) cin
         enddo

         do while (.true.)
            read (1,*, end=300,iostat=ierr) cin
            read (1,*, err=300,iostat=ierr) cin,cin,ntr
            read (1,*, err=300,iostat=ierr) cin,cin,multip
            read (1,*, err=300,iostat=ierr) cin
            read (1,*, err=300,iostat=ierr) cin

            read (1,'(a)') str

c           write (*,*) multip,ii,jj

            if (index(str(76:100),'.').ne.0) then
               uta = .true.
               read (str,*,err=300) ul,gu,ll,gl,dE,Aw,gf,Ar,shalf
               ll  = ll + 1
               ul  = ul + 1
               gll = gl + one
               gul = gu + one
               shalf = shalf/sqrt(two)
               if (phxs) then
                  fout = gf/gll
               else
                  fout = gul*Ar
               endif

               nps1 = nps(iso,ll,:)
               nps2 = nps(iso,ul,:)
               if ((isxtr(nps1,nps2,nsh).ne.0) .and.
     &             ((done(ll,ul).eq.0).or.(abs(multip).eq.1))) then
                  if (dE.gt.zero) then
c                    if ((multip.eq.-1) .and. (Aw.ge.zero)) then
                     if ((shalf.ge.zero) .or. (Aw.ge.zero)) then
                        write (2,950) iso,ll,iso,ul,fout,hc/dE,Aw,shalf
                     elseif (Aw.gt.zero) then
                        write (2,950) iso,ll,iso,ul,fout,hc/dE,Aw
                     else
                        write (2,950) iso,ll,iso,ul,fout,hc/dE
                     endif
                  endif
                  done(ll,ul) = 1
               endif
            else
               uta = .false.
               read (str,*,err=300) ul,gu,ll,gl,dE,gf,Ar,shalf
               Aw  = zero
               ll  = ll + 1
               ul  = ul + 1
               gll = gl + one
               gul = gu + one
               shalf = shalf/sqrt(two)
               if (phxs) then
                  fout = gf/gll
               else
                  fout = gul*Ar
               endif
               nps1 = nps(iso,ll,:)
               nps2 = nps(iso,ul,:)
               if ((isxtr(nps1,nps2,nsh).ne.0) .and.
     &             ((done(ll,ul).eq.0).or.(abs(multip).eq.1))) then
                  if ((dE.gt.zero) .and. (abs(multip).le.multmax)) then
c                    if (multip.eq.-1) then
                     if ((shalf.ge.zero) .or. (Aw.ge.zero)) then
                        write (2,950) iso,ll,iso,ul,fout,hc/dE,Aw,shalf
                     else
                        write (2,950) iso,ll,iso,ul,fout,hc/dE
                     endif
                  endif
                  done(ll,ul) = 1
               endif
            endif

            do i = 2,ntr
               ii = ii+1
               if (uta) then
                  read (1,*,err=300, iostat=ierr) ul,gu,ll,gl,
     &                                            dE,Aw,gf,Ar,shalf

                  if (dE.le.zero) cycle
                  if (shalf.eq.zero) cycle

c                 if (multip.ne.-1) cycle

                  ll  = ll + 1
                  ul  = ul + 1
                  gll = gl + one
                  gul = gu + one
                  shalf = shalf/sqrt(two)
                  if (phxs) then
                     fout = gf/gll
                  else
                     fout = gul*Ar
                  endif

                  nps1 = nps(iso,ll,:)
                  nps2 = nps(iso,ul,:)
                  if ((isxtr(nps1,nps2,nsh).ne.0) .and.
     &                ((done(ll,ul).eq.0).or.(abs(multip).eq.1))) then
                     jj = jj+1
c                    if ((multip.eq.-1) .and. (Aw.ge.zero)) then
                     if ((shalf.ge.zero) .or. (Aw.ge.zero)) then
                        write (2,950) iso,ll,iso,ul,fout,hc/dE,Aw,shalf
                     elseif (Aw.gt.zero) then
                        write (2,950) iso,ll,iso,ul,fout,hc/dE,Aw
                     else
                        write (2,950) iso,ll,iso,ul,fout,hc/dE
                     endif
                     done(ll,ul) = 1
                  else
c                    jj = (2*jj)/2
                  endif
               else
                  read (1,*,err=300, iostat=ierr) ul,gu,ll,gl,
     &                                            dE,gf,Ar,shalf

                  if (dE.le.zero) cycle
                  if (shalf.eq.zero) cycle

                  if (abs(multip).gt.multmax) cycle
c                 if (multip.ne.-1) cycle

                  Aw  = zero
                  ll  = ll + 1
                  ul  = ul + 1
                  gll = gl + one
                  gul = gu + one
                  shalf = shalf/sqrt(two)
                  if (phxs) then
                     fout = gf/gll
                  else
                     fout = gul*Ar
                  endif
                  nps1 = nps(iso,ll,:)
                  nps2 = nps(iso,ul,:)
                  if ((isxtr(nps1,nps2,nsh).ne.0) .and.
     &                ((done(ll,ul).eq.0).or.(abs(multip).eq.1))) then
                     jj = jj+1
c                    if (multip.eq.-1) then
                     if ((shalf.ge.zero) .or. (Aw.ge.zero)) then
                         write (2,950) iso,ll,iso,ul,fout,hc/dE,Aw,shalf
                     else
                         write (2,950) iso,ll,iso,ul,fout,hc/dE
                     endif
                     done(ll,ul) = 1
                  endif
               endif
            enddo
         enddo

 300     close (1)

         if (ierr.gt.0) then
            write (*,*) 'Error # ',ierr,' reading file ',filename
         elseif (ierr.lt.0) then
c           write (*,*) 'Unexpected EOF or EOR in file ',filename
         endif

         deallocate (done)
      enddo

      write (2,910)

c ---------------------------------------------------------------------
c     read & record photoionization/radiative recombination
c ---------------------------------------------------------------------

! Here, cross sections for photoionization in cm^2
! are calculated and stored in the sig arrays.

      if (isoi2.gt.isof) goto 401

      write (2,990)
      write (2,900) 'phis'

      sig0 = two*pi*a0**2*alpha
      do iso = isoi2,isof
         filename = flist(iso,4)
         if (filename.eq.' ') cycle

         write (2,985) els(iso)//'-like'

         write (*,*) '  datafile  ',filename
         open (1,file=filename)

         iferr2 = 0               ! flag for fitting errors
         iferr3 = 0               ! flag for bad fits
         nferr3 = 0

         do while (.true.)
            read (1,*, end=400,iostat=ierr) cin

            if (cin(1:4).eq.'NELE') then
               read (1,*, err=400,iostat=ierr) cin,cin,ntr
            endif

            if (cin(1:4).eq.'UTYP') then
               read (1,*, err=400,iostat=ierr) cin,cin,nP0

               do i = 1,nP0
                  read (1,*, err=400,iostat=ierr) rjnk
               enddo

               do k = 1,ntr
                  x   = one
                  sig = zero
                  cs  = zero

! dE is the ionization energy, lin is the angular momentum of the ionized shell
                  read (1,*,err=400,iostat=ierr) ll,gll,ul0,gul,dE,lin
                  read (1,*,err=400,iostat=ierr) cf1,cf2,cf3,cf4  ! fitting coefficients (see FAC manual)

                  ll  = ll + 1
                  ul  = indst(iso-1,ul0+1-i0(iso-1))
                  gll = gll + one
                  gul = gul + one

                  nP = 0
                  do i = 1,nP0
                     read (1,*,err=400,iostat=ierr) Ef,rjnk,rjnk,csin !read in photoelectron energy Ef (in ev) and d(gf)/dE
                     nP    = nP+1
                     Ew    = dE+Ef         ! photon energy
                     Ewau  = Ew/(two*Ry)   ! photon energy in atomic units
                     Efau  = Ef/(two*Ry)   ! photoelectron energy in atomic units
                     x(nP) = Ew

! sig is photoionization cross section
                     sig(nP) = (sig0*csin/gll)*(one+alpha**2*Efau)
     &                                        /(one+alpha**2*Efau/two)
                  enddo

                  if (Ew/dE.gt.1.d2 .or. cf1.eq.zero) then
                     nP1 = 0
                  else
                     nP1 = 2
                     rfac = exp(log(max(1.d1,1.d2*dE/Ew))/nP1)
                  endif

                  do i = 1,nP1
                     nP    = nP+1
                     Ew    = x(nP-1)*rfac  ! photon energy
                     Ef    = Ew-dE         ! photoelectron energy
                     Ewau  = Ew/(two*Ry)   ! photon energy in atomic units
                     Efau  = Ef/(two*Ry)   ! photoelectron energy in atomic units
                     x(nP) = Ew

                     xx    = one + Ef/cf4
                     yy    = (one + cf3) * sqrt(xx) / (sqrt(xx) + cf3)
                     px    = -3.5d0 - lin

                     csin  = cf1*Ew/(Ef+cf4) * xx**px * yy**cf2

! sig is photoionization cross section
                     sig(nP) = (sig0*csin/gll)*(one+alpha**2*Efau)
     &                                        /(one+alpha**2*Efau/two)
                  enddo
                  emax = x(nP)

                  if (ul.eq.0) cycle

                  if (ll.gt.imax .or. ul.gt.imax) then
                    write (*,*) 'out of bounds index: ',k,ll,ul0
                    write (*,*) cf1,cf2,cf3,cf4
                    stop
                  endif

                  nps1 = nps(iso,ll,:)
                  nps2 = nps(iso-1,ul,:)
                  if (isitr(nps1,nps2,nsh).eq.0) cycle

! fit cross section to log polynomial form
                  call pifit(x,sig,cs,nP,dE,rnorm)

                  if (rnorm.lt.zero) then
                     if (iferr2.eq.0) then
                        write (*,*) 'warning: failures fitting photo',
     &                              ' xsections - see ',efile
                        write (3,*)
                        iferr2 = 1
                     endif
                     write (3,*) 'FAILURE fitting photo xsec'
                     write (3,*) iso,ll,iso-1,ul
                     cycle
                  elseif (rnorm.gt.rmax) then
                     nferr3 = nferr3 + 1
                     if (iferr3.eq.0) then
                        write (*,*) 'warning: bad fits for photo',
     &                              ' xsections - see ',efile
                        write (3,*)
                        iferr3 = 1
                     endif
                     if (rnorm.gt.rmax2) then
                        write (3,*) 'horrible fit for photo xsec: ',
     &                        rnorm,'      transition will be ignored'
                     else
                        write (3,*) 'bad fit for photo xsec: ',rnorm
                     endif
                     write (3,*) iso,ll,iso-1,ul
                  endif

! use matching index indst for the ionized (final) level when printing results

                  if ((rnorm.gt.rmax2) .or. (ul.eq.0)) then
                     write (2,951) iso,ll,iso-1,ul,
     &                  (cs(ii),ii=1,4),one,dE,dE,emax
                  else
                     write (2,950) iso,ll,iso-1,ul,
     &                  (cs(ii),ii=1,4),one,dE,dE,emax
                  endif

! evaluate cross-sections to check fits
                  ng = nGmax
                  if ((ll.le.2 .and. ul.le.2) .or.
     &                (rnorm.gt.rmax .and. nferr3.le.0)) then
                     call pisig(xsig,esig,ng,cs,4,dE,emax)
                     write (4,952) iso,ll,iso-1,ul
                     do ii = 1,nP
                        write (4,954) x(ii),1.d18*sig(ii)
                     enddo
                     write (4,953) 'phis ',iso,ll,iso-1,ul,rnorm
                     do ii = 0,ng
                        write (4,954) esig(ii),xsig(ii)
                     enddo
                  endif
               enddo

            endif
         enddo

 400     close (1)

         if (ierr.gt.0) then
            write (*,*) 'Error # ',ierr,' reading file ',filename
         elseif (ierr.lt.0) then
c           write (*,*) 'Unexpected EOF or EOR in file ',filename
         endif

      enddo

  952 format ('# FAC  ',2x,i3,1x,i4,2x,i3,1x,i4)
  953 format ('# ',a5,2x,i3,1x,i4,2x,i3,1x,i4,4x,1pe10.3,2x,i3)
  954 format (2(1pe15.7))

      write (2,910)

  401 continue

c ---------------------------------------------------------------------
c     read & record collisional excitation
c ---------------------------------------------------------------------

! In the present scheme, dimensionless collision strengths are read in.
! FAC provides two coefficients (c0 and c3) which can be used to fit
! the collision strength (cs) to something like
! cs(x) = c(0) + c(1)/(x+b) + c(2)/(x+b)^2 + c(3)*ln(x), with x=Eimpact/dE.
! collision cross sections sig(x) = Ry*pi*a0^2/g(initial)/Eimpact [cm^2]
! are also calculated in case you want to fit them instead.

      write (2,990)
      write (2,900) 'colex'

      do iso = isoi,isof
         filename = flist(iso,3)

         if (filename.eq.' ') then
            filename = flist(iso,2)
            if ((filename.eq.' ') .or. (.not.addcx)) cycle

            nlev = nliso(iso)
            allocate (done(nlev,nlev))
            done = 0
            goto 510
         endif

         write (2,985) els(iso)//'-like'

         write (*,*) '  datafile  ',filename
         open (1,file=filename)

         iferr2 = 0               ! flag for fitting errors
         iferr3 = 0               ! flag for bad fits
         iferr4 = 0               ! flag for zero fits
         nferr3 = 0

         nlev = nliso(iso)
         allocate (done(nlev,nlev))
         done = 0

         do while (.true.)
            read (1,*, end=500,iostat=ierr) cin

            if (cin(1:4).eq.'NELE') then
               read (1,*, err=500,iostat=ierr) cin,cin,ntr
            endif

            if (cin(1:4).eq.'UTYP') then
               read (1,*, err=500,iostat=ierr) cin,cin,nP0

               do i = 1,nP0
                  read (1,*, err=500,iostat=ierr) rjnk
               enddo

               do k = 1,ntr
                  x   = one
                  sig = zero
                  cs  = zero

                  read (1,*, err=500,iostat=ierr) ll,gll,ul,gul,dE !each transition has a block of data
                  read (1,*, err=500,iostat=ierr) cf1,cf2          !these can be fitting coefficients for collision strength (see FAC manual)
                  if (cf1.eq.-one) then
                     cf1 = zero               !transition is not dipole-allowed
                  endif

                  ll  = ll + 1
                  ul  = ul + 1
                  gll = gll + one
                  gul = gul + one

                  nP = 0
                  do i = 1,nP0
                     read (1,*, err=500,iostat=ierr) Ef,csin                     !read in final electron energy Ef (in ev) and collision strength
                     nP = nP+1
                     x(nP)   = (dE+Ef)/dE
c                    sig(nP) = Ry*pi*a0**2/gll/(dE+Ef)      !collision cross section in cm^2
                     sig(nP) = abs(csin)-cf1*log(x(nP))     !subtract remainder to fit collision strength
                     if (x(nP).gt.xPmax .and. nP.gt.3) nP = nP-1
                  enddo

                  nps1 = nps(iso,ll,:)
                  nps2 = nps(iso,ul,:)
                  if (isxtr(nps1,nps2,nsh).eq.0) cycle

! fit the collision strength less the log term to a suitable form
                  cs(1) = cf1
                  cs(2) = cf2
                  cs(3) = zero
                  cs(4) = zero
                  cs(5) = zero
                  cs(6) = zero

                  call cxfit(x,sig,nP,cs,rnorm,ierr)

                  inone = 1
                  do i = 1,6
                     if (cs(i).gt.zero) inone = 0
                  enddo

                  if (inone.ne.0) then
                     if (iferr4.eq.0) then
                        write (*,*) 'warning: non-positive coll',
     &                              ' strengths - see ',efile
                        write (3,*)
                        iferr4 = 1
                     endif
                     write (3,*) '    non-positive coll strength'
                     write (3,*) iso,ll,iso,ul
                  elseif (ierr.lt.0) then
                     if (iferr2.eq.0) then
                        write (*,*) 'warning: failures fitting coll',
     &                              ' strengths - see ',efile
                        write (*,*) 'substituting Born approximation'
                        write (3,*)
                        iferr2 = 1
                     endif
                     write (3,*) '    FAILURE fitting coll strength'
                     write (3,*) iso,ll,iso,ul
                  elseif (rnorm.gt.rmaxc) then
                     nferr3 = nferr3 + 1
                     if (iferr3.eq.0) then
                        write (*,*) 'warning: bad fits for coll',
     &                              ' strengths - see ',efile
                        write (3,*)
                        iferr3 = 1
                     endif
                     write (3,*) '    bad fit for coll strength: ',rnorm
                     write (3,*) iso,ll,iso,ul
                  endif

c                 write (2,950) iso,ll,iso,ul,(cs(ii),ii=1,6)

! evaluate collision strengths to check fits
                  ng = nGmax
                  if (((ll.le.2 .and. ul.gt.0) .or. (rnorm.lt.zero) .or.
     &                 (rnorm.gt.rmaxc .and. nferr3.le.400))
     &                  .and. (nP.gt.1)) then
                     write (4,952) iso,ll,iso,ul
                     do ii = 1,nP
                        csin = sig(ii) + cs(1)*log(x(ii))
                        write (4,954) x(ii),csin
                     enddo
                     write (4,953) 'colex',iso,ll,iso,ul,rnorm,ierrc
                     do ii = 0,ng
                        esig(ii) = x(1) + ii*(x(nP)-x(1))/ng
                        xsig(ii) = cs(1)*log(esig(ii)) + cs(2) +
     &                             cs(3)/(esig(ii)+cs(5)) +
     &                             cs(4)/(esig(ii)+cs(5))**2
                        write (4,954) esig(ii),xsig(ii)
                     enddo
c                     write (4,952) iso,ll,iso,ul
c                     do ii = 1,nP
c                        csin = sig(ii)
c                        write (4,954) x(ii),csin
c                     enddo
c                     write (4,953) 'colex',iso,ll,iso,ul,rnorm,ierrc
c                     do ii = 0,ng
c                        esig(ii) = x(1) + ii*(x(nP)-x(1))/ng
c                        xsig(ii) = cs(2) +
c     &                             cs(3)/(esig(ii)+cs(5)) +
c     &                             cs(4)/(esig(ii)+cs(5))**2
c                        write (4,954) esig(ii),xsig(ii)
c                     enddo
                  endif

! fall back to oscillator strengths if the fit is too bad
!   - first need to check if an oscillator strength exists
c                  if (rnorm.lt.rmaxc) then
                     write (2,950) iso,ll,iso,ul,(cs(ii),ii=1,6)
                     done(ll,ul) = 1
c                  endif

               enddo
            endif
         enddo

 500     close (1)

         if (ierr.gt.0) then
            write (*,*) 'Error # ',ierr,' reading file ',filename
         elseif (ierr.lt.0) then
c           write (*,*) 'Unexpected EOF or EOR in file ',filename
         endif

c ... now augment these with rates built off photoexcitations

  510    filename = flist(iso,2)
         if (filename.eq.' ') then
            cycle
         endif

         write (*,*) '  datafile  ',filename
         open (1,file=filename)

         do i = 1,7
            read (1,*, err=550,iostat=ierr) cin
         enddo

         cs = zero
         do while (.true.)
            read (1,*, end=550,iostat=ierr) cin
            read (1,*, err=550,iostat=ierr) cin,cin,ntr
            read (1,*, err=550,iostat=ierr) cin,cin,multip
            read (1,*, err=550,iostat=ierr) cin
            read (1,*, err=550,iostat=ierr) cin

            if (multip.eq.-1) then
               multip = 1
            elseif (multip.eq.-2) then
               multip = 2
            else
               multip = 3
            endif

            read (1,'(a)') str

            if (index(str(76:100),'.').ne.0) then
               uta = .true.
               read (str,*,err=550) ul,gu,ll,gl,dE,Aw,gf,Ar,shalf
            else
               uta = .false.
               read (str,*,err=550) ul,gu,ll,gl,dE,gf,Ar,shalf
            endif

            ll  = ll + 1
            ul  = ul + 1
            gll = gl + one
            gul = gu + one

            fout = gf/gll

            dE = E(iso,ul) - E(iso,ll)

            nps1 = nps(iso,ll,:)
            nps2 = nps(iso,ul,:)
            if (isxtr(nps1,nps2,nsh).ne.0) then
               if ((dE.gt.1.d-4*Ry) .and. (done(ll,ul).eq.0)) then
                  call cxval(dE/Ry,fout,gll,gul,multip,cs(1),cs(2))
                  write (2,950) iso,ll,iso,ul,(cs(ii),ii=1,6)
                  done(ll,ul) = 1
               endif
            endif

            do i = 2,ntr
               if (uta) then
                  read (1,*,err=550, iostat=ierr) ul,gu,ll,gl,
     &                                            dE,Aw,gf,Ar,shalf
                  if (dE.le.zero) cycle
               else
                  read (1,*,err=550, iostat=ierr) ul,gu,ll,gl,
     &                                            dE,gf,Ar,shalf
                  if (dE.le.zero) cycle
               endif

               ll  = ll + 1
               ul  = ul + 1
               gll = gl + one
               gul = gu + one

               fout = gf/gll

               dE = E(iso,ul) - E(iso,ll)

               nps1 = nps(iso,ll,:)
               nps2 = nps(iso,ul,:)
               if (isxtr(nps1,nps2,nsh).ne.0) then
                  if ((dE.gt.1.d-4*Ry) .and. (done(ll,ul).eq.0)) then
                     call cxval(dE/Ry,fout,gll,gul,multip,cs(1),cs(2))
                     write (2,950) iso,ll,iso,ul,(cs(ii),ii=1,6)
                     done(ll,ul) = 1
                  endif
               endif
            enddo
         enddo

 550     close (1)

 560     deallocate (done)
      enddo

      write (2,910)

c ---------------------------------------------------------------------
c     read & record collisional ionization
c ---------------------------------------------------------------------

! Similar to collisional excitation, dimensionless collision strengths are read in.
! FAC provides the coefficient c3 which can be used in the fitting:
! cs(x) = c(0) + c(1)/(x) + c(2)/(x)^2 + c(3)*ln(x), with x=Eimpact/dE.
! collision cross sections sig(x) = Ry*pi*a0^2/g(initial)/Eimpact [cm^2]
! are also calculated in case you want to fit them instead.

      if (isoi2.gt.isof) goto 701

      write (2,990)
      write (2,900) 'colon'

      do iso = isoi2,isof
         filename = flist(iso,6)
         if (filename.eq.' ') cycle

         write (2,985) els(iso)//'-like'

         write (*,*) '  datafile  ',filename
         open (1,file=filename)

         do while (.true.)
            read (1,*, end=600,iostat=ierr) cin

            if (cin(1:4).eq.'NELE') then
               read (1,*, err=600,iostat=ierr) cin,cin,ntr
            endif

            if (cin(1:4).eq.'UTYP') then
               read (1,*, err=600,iostat=ierr) cin,cin,nP0

               do i = 1,nP0
                  read (1,*, err=600,iostat=ierr) rjnk
               enddo

               do k = 1,ntr
                  x   = one
                  sig = zero
                  cs  = zero

                  read (1,*, err=600,iostat=ierr) ll,gll,ul0,gul,dE !each transition has a block of data
                  read (1,*, err=600,iostat=ierr) cf1,cf2,cf3,cf4   !these can be fitting coefficients for collision strength (see FAC manual)
                  cf1 = cf1/pi                 !factor of 1/pi
                  cf2 = cf2/pi                 !factor of 1/pi
                  cf3 = cf3/pi                 !factor of 1/pi
                  cf4 = cf4/pi                 !factor of 1/pi

                  ll  = ll + 1
                  ul  = indst(iso-1,ul0+1-i0(iso-1))
                  gll = gll + one
                  gul = gul + one

                  nP = 0
                  do i = 1,nP0
                     read (1,*, err=600,iostat=ierr) Ef,csin !read in final electron energy Ef (in ev) and collision strength
                     nP = nP+1
                     x(nP)   = (dE+Ef)/dE
                     sig(nP) = Ry*a0**2/gll/(dE+Ef)    !collision cross section in cm^2
                     cs(nP)  = dabs(csin)/pi - cf1*dlog(x(nP)) !subtract remainder to fit collision strength
                  enddo

! change the fitting parameters for the collision strength
! to match those expected by Cretin
                  cs(1) = cf1
                  cs(2) = cf2
                  cs(3) = cf3 - 2*cf2
                  cs(4) = cf4 - cf3 + cf2
                  cs(5) = zero
                  cs(6) = -cf4

                  if (ul.eq.0) cycle

                  nps1 = nps(iso,ll,:)
                  nps2 = nps(iso-1,ul,:)
                  if (isitr(nps1,nps2,nsh).ne.0) then
                     if (ul.eq.0) then
                        write (2,951) iso,ll,iso-1,ul,(cs(ii),ii=1,6)
                     else
                        write (2,950) iso,ll,iso-1,ul,(cs(ii),ii=1,6)
                     endif
                  endif
               enddo

            endif
         enddo

 600     close (1)

         if (ierr.gt.0) then
            write (*,*) 'Error # ',ierr,' reading file ',filename
         elseif (ierr.lt.0) then
c           write (*,*) 'Unexpected EOF or EOR in file ',filename
         endif

      enddo

      write (2,910)

c ---------------------------------------------------------------------
c     read & record Auger decay
c ---------------------------------------------------------------------

      write (2,990)
      write (2,900) 'augxs'

      do iso = isoi2,isof
         if (maxval(E(iso,1:imax)).gt.Ez(iso)) then !autoionizing states exist

            filename = flist(iso,5)
            if (filename.eq.' ') cycle

            write (2,985) els(iso)//'-like'

            write (*,*) '  datafile  ',filename
            open (1,file=filename)

            do i = 1,7
               read (1,*, err=700,iostat=ierr) cin
            enddo

            do while (.true.)
               read (1,*, end=700,iostat=ierr) cin
               read (1,*, err=700,iostat=ierr) cin,cin,ntr
               read (1,*, err=700,iostat=ierr) cin
               read (1,*, err=700,iostat=ierr) cin, cin,jj
               do i = 1,jj
                 read (1,*, err=700,iostat=ierr) cin
               enddo

               do i = 1,ntr
                  read (1,*, err=700,iostat=ierr) ul,gu,ll,gl,rjnk,Ar
                  ll  = ll + 1 - i0(iso-1)
                  ul  = ul + 1
                  write (2,950) iso,ul,iso-1,ll,Ar
               enddo
            enddo

  700       close (1)

            if (ierr.gt.0) then
               write (*,*) 'Error # ',ierr,' reading file ',filename
            elseif (ierr.lt.0) then
c              write (*,*) 'Unexpected EOF or EOR in file ',filename
            endif

         endif
      enddo

      write (2,910)

  701 continue

      close (2)
      close (3)
      close (4)

      end program

c *********************************************************************
c *********************************************************************

      integer function isxtr(nps1,nps2,nsh)

c ... compare shell occupations to confirm an excitation which changes
c ... the PQN of a single electron or is a deltaN=0 transition

c ... this assumes shells denoting (n,l)

      implicit none

      integer  nsh
      integer  nps1(nsh), nps2(nsh)

      integer  is
      integer  n1, n2, ndp, nns

c ---------------------------------------------------------------------

      isxtr = 0

      n1 = 0
      n2 = 0

      call nps2nns(nps1,nsh,nns)
      call nps2nns(nps2,nsh,nns)

      do is = 1,nns
         ndp = nps1(is) - nps2(is)

         if (ndp.eq.1) then
            if (n1.eq.0) then
               n1 = is
            else
               return
            endif
         elseif (ndp.eq.-1) then
            if (n2.eq.0) then
               n2 = is
            else
               return
            endif
         elseif (ndp.ne.0) then
            return
         endif
      enddo

      if ((n1.ne.0) .and. (n2.ne.0)) then
         isxtr = 1
      elseif ((n1.eq.0) .and. (n2.eq.0)) then
         isxtr = 1
      endif

      return
      end

c *********************************************************************
c *********************************************************************

      integer function isitr(nps1,nps2,nsh)

c ... compare shell occupations to confirm an ionization which decreases
c ... the occupation of one shell by a single electron

c ... this assumes shells denoting (n,l)

      implicit none

      integer  nsh
      integer  nps1(nsh), nps2(nsh)

      integer  is
      integer  n1, ndp, nns

c ---------------------------------------------------------------------

      isitr = 0

      n1 = 0

      call nps2nns(nps1,nsh,nns)
      call nps2nns(nps2,nsh,nns)

      do is = 1,nns
         ndp = abs(nps1(is) - nps2(is))

         if (ndp.eq.1) then
            if (n1.eq.0) then
               n1 = is
            else
               return
            endif
         elseif (ndp.ne.0) then
            return
         endif
      enddo

      if (n1.ne.0) then
         isitr = 1
      endif

      return
      end

c *********************************************************************
c *********************************************************************

      subroutine nps2nns(nps,nsh,n)

c ... change shell occupations to PQN occupations

      implicit none

      integer  nsh, n
      integer  nps(nsh)

      integer  i, imin, imax

c ---------------------------------------------------------------------

      n = 1
      if (nsh.le.2) return

  100 n = n + 1

      if (n.eq.2) then       ! avoid overwriting nps(2)
         imin = 3
         imax = 3
      else
         imin = n*(n-1)/2 + 1
         imax = n*(n+1)/2
         nps(n) = 0
      endif
      do i = imin,imax
         nps(n) = nps(n) + nps(i)
         nps(i) = 0
         if (i.eq.nsh) return
      enddo

      goto 100

      return
      end

c *********************************************************************
c *********************************************************************

      subroutine cxval(de,fosc,gi,gj,type,f0,f1)

c ... calculate modified Van-Regemorter collision strength
c ... coefficients as done in SCRAM

c ... the collision strength = gaunt factor * pi83/de

c ... input :  de    = threshold energy in units of Rdybergs
c              fosc  = oscillator strength
c              gi,gj = statistical weights of lower,upper states
c              type  = 1 (E1), 2 (E2) or 3 (other)

c ... output:  f0, f1

      use cdat
      implicit  none

      double precision  de, fosc, f0, f1, gi, gj
      integer           type

      double precision  cval

      double precision  pi83
      parameter (pi83  = 14.5103949d0)   ! 8*pi/sqrt(3)

c ---------------------------------------------------------------------

      f0 = zero
      f1 = zero

      cval = pi83*fosc/de

      if (type.eq.1) then                    ! Mewe for E1
         f0 = 0.28d0*cval
         f1 = 0.15d0*cval
      elseif (type.eq.2) then                ! fit to DW for E2
         f1 = 1.2d2*cval
      else                                   ! Mx
         f1 = 1.d-3/gj
         if (abs(gi-gj).gt.1.d0) then
            f1 = 0.1d0*f1
         endif
      endif

      return
      end

c *********************************************************************
c *********************************************************************

      subroutine filelist(flist,nlist,esym,niso,nlev,nmax)

!**********************************************************************
!
!     scan directory for FAC files
!
!**********************************************************************

      implicit none





      integer       nlist, niso, nlev, nmax
      character*32  flist(niso,0:11)
      character*(*) esym

      integer       i, ierr, ilev, imax, iso, itype
      character*32  filename, word, word2, word3

c ---------------------------------------------------------------------

      esym = ' '
      nlev = 0
      nmax = 0




      call system('ls > dir.list')


      open (1,file='dir.list')

  100 read (1,*,end=130) filename

      if (filename.eq.'dir.list') goto 100

         open (2,file=filename,err=110,iostat=ierr)

         read (2,*,end=120,err=120,iostat=ierr) word
         if (word.ne.'FAC') goto 120

         do i = 1,2
            read (2,*,err=110,end=110,iostat=ierr) word
         enddo

         read (2,*,err=110,end=110,iostat=ierr) word, word, itype

         read (2,*,err=110,end=110,iostat=ierr) word

         read (2,*,err=110,end=110,iostat=ierr) word

         if (esym.eq.' ') then
            esym = word
         elseif (word.ne.esym) then
         endif

         read (2,*,err=110,end=110,iostat=ierr) word
         if (itype.eq.1) then
            read (2,*,err=110,end=110,iostat=ierr) word
         endif

         read (2,*,err=110,end=110,iostat=ierr) word, word, iso

         if (itype.eq.1) then
            imax = 0
            do while (.true.)
               read (2,*,err=110,end=105,iostat=ierr) word, word2, word3
               if (word.eq.'NELE') then
                  read (word3,*) ilev
                  if (ilev.ne.iso) goto 105
               elseif (word.eq.'NLEV') then
                  read (word3,*) ilev
                  read (2,*,err=110,end=110,iostat=ierr) word
                  imax = imax + ilev
                  nlev = nlev + ilev
                  nmax = max(nmax,imax)
               endif
            enddo
         endif

  105    if ((iso.gt.0)   .and. (iso.le.niso) .and.
     &       (itype.gt.0) .and. (itype.le.11)) then
            if ((itype.eq.1) .and. (flist(iso,itype).ne.' ')) itype = 0
            flist(iso,itype) = filename
            nlist = nlist + 1
         endif

         if (ierr.lt.0) goto 120

  110    if (ierr.gt.0) then
            write (*,*) 'Error # ',ierr,' reading file ',filename
         elseif (ierr.lt.0) then
            write (*,*) 'Unexpected EOF or EOR in file ',filename
         endif

  120    close (2)

      goto 100

  130 close (1)




      call system('rm dir.list')


      write (*,*) nlist, 'valid FAC files found for ',esym
      write (*,*) nlev, 'energy levels'
      write (*,*) nmax, 'max levels per sequence'

      if (nlist.eq.0) stop

      return
      end

c *********************************************************************
c *********************************************************************

      subroutine setdat

!**********************************************************************
!
!     elemental names and symbols
!
!**********************************************************************

      use cdat

c ---------------------------------------------------------------------

      elname = (/
     &   'bare        ',
     &   'Hydrogen    ', 'Helium      ', 'Lithium     ', 'Beryllium   ',
     &   'Boron       ', 'Carbon      ', 'Nitrogen    ', 'Oxygen      ',
     &   'Fluorine    ', 'Neon        ', 'Sodium      ', 'Magnesium   ',
     &   'Aluminum    ', 'Silicon     ', 'Phosphorus  ', 'Sulfur      ',
     &   'Chlorine    ', 'Argon       ', 'Potassium   ', 'Calcium     ',
     &   'Scandium    ', 'Titanium    ', 'Vanadium    ', 'Chromium    ',
     &   'Manganese   ', 'Iron        ', 'Cobalt      ', 'Nickel      ',
     &   'Copper      ', 'Zinc        ', 'Gallium     ', 'Germanium   ',
     &   'Arsenic     ', 'Selenium    ', 'Bromine     ', 'Krypton     ',
     &   'Rubidium    ', 'Strontium   ', 'Yttrium     ', 'Zirconium   ',
     &   'Niobium     ', 'Molybdenum  ', 'Technetium  ', 'Ruthenium   ',
     &   'Rhodium     ', 'Palladium   ', 'Silver      ', 'Cadmium     ',
     &   'Indium      ', 'Tin         ', 'Antimony    ', 'Tellurium   ',
     &   'Iodine      ', 'Xenon       ', 'Cesium      ', 'Barium      ',
     &   'Lanthanum   ', 'Cerium      ', 'Praseodymium', 'Neodymium   ',
     &   'Promethium  ', 'Samarium    ', 'Europium    ', 'Gadolinium  ',
     &   'Terbium     ', 'Dysprosium  ', 'Holmium     ', 'Erbium      ',
     &   'Thulium     ', 'Ytterbium   ', 'Lutetium    ', 'Hafnium     ',
     &   'Tantalum    ', 'Tungsten    ', 'Rhenium     ', 'Osmium      ',
     &   'Iridium     ', 'Platinum    ', 'Gold        ', 'Mercury     ',
     &   'Thallium    ', 'Lead        ', 'Bismuth     ', 'Polonium    ',
     &   'Astatine    ', 'Radon       ', 'Francium    ', 'Radium      ',
     &   'Actinium    ', 'Thorium     ', 'Protactinium', 'Uranium     ',
     &   'Neptunium   ', 'Plutonium   ', 'Americium   ', 'Curium      ',
     &   'Berkelium   ', 'Californium ', 'Einsteinium ' /)

      els = (/
     &   '__',
     &   '_H', 'He', 'Li', 'Be', '_B', '_C', '_N', '_O',
     &   '_F', 'Ne', 'Na', 'Mg', 'Al', 'Si', '_P', '_S',
     &   'Cl', 'Ar', '_K', 'Ca', 'Sc', 'Ti', '_V', 'Cr',
     &   'Mn', 'Fe', 'Co', 'Ni', 'Cu', 'Zn', 'Ga', 'Ge',
     &   'As', 'Se', 'Br', 'Kr', 'Rb', 'Sr', '_Y', 'Zr',
     &   'Nb', 'Mo', 'Tc', 'Ru', 'Rh', 'Pd', 'Ag', 'Cd',
     &   'In', 'Sn', 'Sb', 'Te', '_I', 'Xe', 'Cs', 'Ba',
     &   'La', 'Ce', 'Pr', 'Nd', 'Pm', 'Sm', 'Eu', 'Gd',
     &   'Tb', 'Dy', 'Ho', 'Er', 'Tm', 'Yb', 'Lu', 'Hf',
     &   'Ta', '_W', 'Re', 'Os', 'Ir', 'Pt', 'Au', 'Hg',
     &   'Tl', 'Pb', 'Bi', 'Po', 'At', 'Rn', 'Fr', 'Ra',
     &   'Ac', 'Th', 'Pa', '_U', 'Np', 'Pu', 'Am', 'Cm',
     &   'Bk', 'Cf', 'Es' /)

      end subroutine

c *********************************************************************
c *********************************************************************

      subroutine pifit(efit,sfit,rfit,nfit,de,rnorm)

      implicit none

      real*8 ryd0
      parameter (ryd0 = 13.606d0)

      integer mcf, ncf
      parameter (mcf = 4)

      integer nfit, mwrk
      real*8  efit(nfit), sfit(nfit), rfit(nfit)
      real*8  afit(nfit,mcf), wrk(2*nfit*mcf)

      integer i, j, ierr
      real*8  de, rnorm

      real*8  zero, one, two
      parameter (zero = 0.d0, one = 1.d0, two = 2.d0)

c ---------------------------------------------------------------------

      ierr = 0

      if (de.le.zero) then
         ierr = -1
      endif

      do 90 i = 1,nfit
         if (efit(i).le.zero) ierr = -1
         if (sfit(i).le.zero) ierr = -1
   90 continue

      if (ierr.lt.0) goto 220

      ncf = min(mcf,nfit)

      do 100 i = 1,nfit
         efit(i) = log(efit(i)/de)
         rfit(i) = log(1.d18*sfit(i)*de/ryd0)
  100 continue

      do 210 i = 1,nfit
         afit(i,1) = one
         do 200 j = 2,ncf
            afit(i,j) = afit(i,j-1)*efit(i)
  200    continue
  210 continue

      mwrk = 2*nfit*mcf
      call dgels('N', nfit,ncf,1, afit,nfit, rfit,nfit, wrk,mwrk, ierr)

  220 if (ierr.lt.0) then
         rnorm = -one
      else
         rnorm = zero
         do 230 i = ncf+1,nfit
            rnorm = rnorm + rfit(i)**2
  230    continue
         if (nfit.gt.mcf) rnorm = sqrt(rnorm/(nfit-mcf))
      endif

      do 300 i = 1,nfit
         efit(i) = de*exp(efit(i))
  300 continue

      return
      end

c *********************************************************************
c *********************************************************************

      subroutine pisig(xsig,esig,ngp,sfit,nfit,de,emax)

      implicit none

      real*8 ryd0
      parameter (ryd0 = 13.606d0)

      integer ngp, nfit
      real*8  xsig(0:ngp), esig(0:ngp), sfit(nfit)
      real*8  de, emax

      integer igp
      real*8  x, sig0

c ---------------------------------------------------------------------

      do igp = 0,ngp
         esig(igp) = de + igp*(emax-de)/ngp
         x    = log(esig(igp)/de)
         sig0 = sfit(1) + sfit(2)*x + sfit(3)*x**2 + sfit(4)*x**3
         xsig(igp) = exp(sig0)*ryd0/de
      enddo

      return
      end

c *********************************************************************
c *********************************************************************

      subroutine cxfit(xfit,sfit,nfit,cval,rnorm,ierr)

c ... fit collisional excitation collision strengths to the form
c ...    omega = c0*log(x) + c1 + c2/(x+dx) + c3/(x+dx)**2
c ... where x = E / Ethreshold

c ... c0 is not changed but is used to check properties of the fit

c ... Input:
c ...    cval(0) = c0
c ...    xfit(i) = x
c ...    sfit(i) = omega - c0*log(x)

c ... Output:
c ...    cval(1) = c1
c ...    cval(2) = c2
c ...    cval(3) = c3
c ...    cval(4) = dx

      implicit none

      integer mfit
      parameter (mfit = 30)

      integer ncf
      parameter (ncf = 4)

      integer mcf, mwrk, miwrk
      parameter (mcf = 6, mwrk = mfit*(mcf+5)+mfit, miwrk = mfit)

      real*8  omg(mfit), xval(mfit)
      common /sval/ omg, xval

      integer nfit
      real*8  xfit(nfit), sfit(nfit), cval(0:ncf)
      real*8  rnorm

      integer iwrk(miwrk)
      real*8  fvec(mfit), fjac(mfit*mcf), wrk(mwrk)

      integer i, j, ierr, iopt

      external cxfcn, cxfcn2, cxfcnb

      real*8  cf1, cvals, denorm, omin, omax, xv

      real*8  tol, rmax
      parameter (tol = 1.d-6, rmax = 1.d2)

      real*8  zero, one, two
      parameter (zero = 0.d0, one = 1.d0, two = 2.d0)

c ---------------------------------------------------------------------

      ierr = 0
      if (nfit.lt.ncf-1) return

      if (nfit.gt.mfit) stop

      cf1 = cval(1)

      omin = zero
      omax = zero
      do i = 1,nfit
         xval(i) = xfit(i)
         omg(i)  = sfit(i)
         omin = min(omin,omg(i))
         omax = max(omax,omg(i))
      enddo

c ... shift all strengths to remain positive
      do i = 1,nfit
         omg(i) = omg(i) - omin
      enddo

c ... starting guess for fit parameters matches data at 1 & nfit
      cval(2) = (omg(nfit)*xval(nfit)*2 - omg(1)*xval(1)**2) /
     &          (xval(nfit)**2 - xval(1)**2)
     &         - cval(1)*(xval(nfit)+xval(1))
      cval(3) = (omg(1)*xval(1) - omg(nfit)*xval(nfit))*
     &           xval(1)*xval(nfit) / (xval(nfit) - xval(1))
     &         + cval(1)*xval(nfit)*xval(1)
      cval(4) = zero

      call xsetun(3)
      call xsetf(-1)

c ... try for fitting all 4 parameters if there are at least 4 points
c ... otherwise try without the constant term (if zero)
c ... otherwise try without the denominator term

c     iopt = 2
      iopt = 1
      if (nfit.ge.ncf) then
         call dnls1e(cxfcn, iopt, nfit, ncf, cval(1), fvec, tol*omax,
     &               0, ierr, iwrk, wrk, mwrk)
      elseif (cf1.eq.zero) then
         call dnls1e(cxfcn2, iopt, nfit, ncf-1, cval(2), fvec, tol*omax,
     &               0, ierr, iwrk, wrk, mwrk)
      else
         call dnls1e(cxfcnb, iopt, nfit, ncf-1, cval(1), fvec, tol,
     &               0, ierr, iwrk, wrk, mwrk)
      endif

c ... ierr = 0 => improper input parameters
c ... ierr = 1 => relative error in sum of squares <= tol
c ... ierr = 2 => relative error in solution <= tol
c ... ierr = 3 => both (1) and (2)
c ... ierr = 4 => fvec orthogonal to jacobian to machine precision
c ... ierr = 5 => too many function calls
c ... ierr = 6 => tol too small to improve sum of squares
c ... ierr = 7 => tol too small to improve solution

      if (ierr.eq.0) then
         write (*,*) 'Improper input parameters to DNLS1E'
         stop
      else
         rnorm = denorm(nfit,fvec)
      endif

c ... redo the case where the denominator can go negative with
c ... cval(4) = zero
      if (cval(4).le.-one) then
         cval(4) = zero

         call dnls1e(cxfcnb, iopt, nfit, ncf-1, cval(1), fvec, tol,
     &               0, ierr, iwrk, wrk, mwrk)
      endif

c ... flag this because it might denote NaN's being return
      if (ierr.eq.4) then
         ierr = -1
      endif

c ... flag the case where the fitting coefficients are large and
c ... alternating in sign
      if ((abs(cval(2)).gt.rmax*omax) .and.
     &    (abs(cval(3)).gt.rmax*omax) .and.
     &    (cval(2)*cval(3).lt.zero)) then
c           ierr = -2
      endif

c ... flag the case where the value at xmin is negative
      cvals = cval(0)*log(xfit(1)) +
     &        cval(1) + cval(2)/(xfit(1)+cval(4))
     &                + cval(3)/(xfit(1)+cval(4))**2

      if (cvals.lt.zero) then
         ierr = -4
      endif

c ... flag the case where the slope at x=1 is negative
c     cvals = cval(0) - cval(2)/(one+cval(4))**2
c    &                - two*cval(3)/(one+cval(4))**3

c     if (cvals.lt.zero) then
c        ierr = -5
c     endif

c ... flag the case where the slope at x=10*xmax is positive
c ... (not including the contribution from the log term)
c     xv = 10.d0*xfit(nfit)
c     cvals = - cval(2)/(xv+cval(4))**2
c    &        - two*cval(3)/(xv+cval(4))**3

c     if (cvals.gt.zero) then
c        ierr = -6
c     endif

      if (ierr.lt.0) then
         if (cval(0).ne.zero) then
            cval(1) = cf1
         elseif (cf1.gt.zero) then
            cval(1) = cf1
         else
            cval(1) = sfit(1)*xfit(1)
         endif
         cval(2) = zero
         cval(3) = zero
         cval(4) = zero
      endif

      if (ierr.lt.zero) then
         rnorm = zero
         do i = 1,nfit
            cvals = cval(0)*log(xfit(i)) +
     &              cval(1) + cval(2)/(xfit(i)+cval(4))
     &                      + cval(3)/(xfit(i)+cval(4))**2
            rnorm = rnorm + (cvals-sfit(i))**2/sfit(i)**2
         enddo
         rnorm = sqrt(rnorm)/nfit
      endif

      return
      end

c *********************************************************************
c *********************************************************************

      subroutine cxfit2(xfit,sfit,nfit,cval,rnorm,ierr)

c ... fit collisional excitation collision strengths to the form
c ...    omega = c0*log(x) + c1 + c2/(x+dx) + c3/(x+dx)**2
c ... where x = E / Ethreshold

c ... c0 and c1 are not changed

c ... Input:
c ...    cval(0) = c0
c ...    cval(1) = c1
c ...    xfit(i) = x
c ...    sfit(i) = omega - c0*log(x)

c ... Output:
c ...    cval(1) = c2
c ...    cval(2) = c3
c ...    cval(3) = dx

      implicit none

      integer mfit
      parameter (mfit = 30)

      integer ncf
      parameter (ncf = 3)

      integer mcf, mwrk, miwrk
      parameter (mcf = 6, mwrk = mfit*(mcf+5)+mfit, miwrk = mfit)

      real*8  omg(mfit), xval(mfit)
      common /sval/ omg, xval

      integer nfit
      real*8  xfit(nfit), sfit(nfit), cval(0:ncf+1)
      real*8  rnorm

      integer iwrk(miwrk)
      real*8  fvec(mfit), fjac(mfit*mcf), wrk(mwrk)

      integer i, j, ierr, iopt

      external cxfcn2, cxfcn2b

      real*8  cf1, cvals, denorm, omin, omax, xv

      real*8  tol, rmax
      parameter (tol = 1.d-6, rmax = 1.d2)

      real*8  zero, one, two
      parameter (zero = 0.d0, one = 1.d0, two = 2.d0)

c ---------------------------------------------------------------------

      ierr = 0
      if (nfit.lt.ncf-1) return

      if (nfit.gt.mfit) stop

      cf1 = cval(1)

      omin = zero
      omax = zero
      do i = 1,nfit
         xval(i) = xfit(i)
         omg(i)  = sfit(i) - cf1
         omin = min(omin,omg(i))
         omax = max(omax,omg(i))
      enddo

c ... shift all strengths to remain positive
      do i = 1,nfit
         omg(i) = omg(i) - omin
      enddo

c ... starting guess for fit parameters
      cval(2) = omg(1)*xval(1)
      cval(3) = sqrt(omg(nfit)*xval(nfit))
      cval(4) = zero

      call xsetun(3)
      call xsetf(-1)

      iopt = 2
      if (nfit.ge.ncf-1) then
         call dnls1e(cxfcn2, iopt, nfit, ncf-1, cval(2), fvec, tol,
     &               0, ierr, iwrk, wrk, mwrk)
      else
         call dnls1e(cxfcn2b, iopt, nfit, ncf-2, cval(2), fvec, tol,
     &               0, ierr, iwrk, wrk, mwrk)
      endif

c ... ierr = 0 => improper input parameters
c ... ierr = 1 => relative error in sum of squares <= tol
c ... ierr = 2 => relative error in solution <= tol
c ... ierr = 3 => both (1) and (2)
c ... ierr = 4 => fvec orthogonal to jacobian to machine precision
c ... ierr = 5 => too many function calls
c ... ierr = 6 => tol too small to improve sum of squares
c ... ierr = 7 => tol too small to improve solution

      if (ierr.eq.0) then
         write (*,*) 'Improper input parameters to DNLS1E'
         stop
      else
         rnorm = denorm(nfit,fvec)
      endif

c ... redo the case where the denominator can go negative
      if (cval(4).le.-one) then
         cval(4) = zero

         call dnls1e(cxfcn2b, iopt, nfit, ncf-2, cval(2), fvec, tol,
     &               0, ierr, iwrk, wrk, mwrk)
      endif

c ... flag this because it might denote NaN's being return
      if (ierr.eq.4) then
         ierr = -1
      endif

c ... flag the case where the fitting coefficients are large and
c ... alternating in sign
      if ((abs(cval(2)).gt.rmax*omax) .or.
     &    (abs(cval(3)).gt.rmax*omax)) then
         if (cval(2)*cval(3).lt.zero) then
            ierr = -2
         endif
      endif

c ... flag the case where the value at xmin is negative
      cvals = cval(0)*log(xfit(1)) +
     &        cval(1) + cval(2)/(xfit(1)+cval(4))
     &                + cval(3)/(xfit(1)+cval(4))**2

      if (cvals.lt.zero) then
         ierr = -4
      endif

c ... flag the case where the slope at x=1 is negative
c     cvals = cval(0) - cval(2)/(one+cval(4))**2
c    &                - two*cval(3)/(one+cval(4))**3

c     if (cvals.lt.zero) then
c        ierr = -5
c     endif

c ... flag the case where the slope at x=10*xmax is positive
c ... (not including the contribution from the log term)
      xv = 10.d0*xfit(nfit)
      cvals = - cval(2)/(xv+cval(4))**2
     &        - two*cval(3)/(xv+cval(4))**3

      if (cvals.gt.zero) then
         ierr = -6
      endif

c      if (ierr.lt.0) then
c         if (cval(2).lt.zero) then
c            cval(2) = zero
c         elseif (cval(3).lt.zero) then
c            cval(3) = zero
c         endif
c      endif

      if (ierr.lt.zero) then
         rnorm = zero
         do i = 1,nfit
            cvals = cval(0)*log(xfit(i)) +
     &              cval(1) + cval(2)/(xfit(i)+cval(4))
     &                      + cval(3)/(xfit(i)+cval(4))**2
            rnorm = rnorm + (cvals-sfit(i))**2/sfit(i)**2
         enddo
         rnorm = sqrt(rnorm)/nfit
      endif

      return
      end

c *********************************************************************
c *********************************************************************

      subroutine cxfcn(iflag,m,n,x,fvec,fjac,ldfjac)

      implicit none

      integer m, n, ldfjac, iflag
      real*8  x(n), fvec(m), fjac(ldfjac,n)

      integer i

      integer mfit
      parameter (mfit = 30)

      real*8  omg(mfit), xval(mfit)
      common /sval/ omg, xval

      real*8  zero, one, two
      parameter (zero = 0.d0, one = 1.d0, two = 2.d0)

c ---------------------------------------------------------------------

      if (iflag.eq.1) then

         do i = 1,m
            fvec(i) = omg(i) - x(1) - x(2)/(xval(i)+x(4))
     &                       - x(3)/(xval(i)+x(4))**2
         enddo

      elseif (iflag.eq.2) then

         do i = 1,m
            fjac(i,1) = -one
            fjac(i,2) = -one/(xval(i)+x(4))
            fjac(i,3) = -one/(xval(i)+x(4))**2
            fjac(i,4) =     x(2)/(xval(i)+x(4))**2 +
     &                  two*x(3)/(xval(i)+x(4))**3
         enddo

      endif

      return
      end

c *********************************************************************
c *********************************************************************

      subroutine cxfcnb(iflag,m,n,x,fvec,fjac,ldfjac)

      implicit none

      integer m, n, ldfjac, iflag
      real*8  x(n), fvec(m), fjac(ldfjac,n)

      integer i

      integer mfit
      parameter (mfit = 30)

      real*8  omg(mfit), xval(mfit)
      common /sval/ omg, xval

      real*8  zero, one, two
      parameter (zero = 0.d0, one = 1.d0, two = 2.d0)

c ---------------------------------------------------------------------

      if (iflag.eq.1) then

         do i = 1,m
            fvec(i) = omg(i) - x(1) - x(2)/xval(i)
     &                       - x(3)/xval(i)**2
         enddo

      elseif (iflag.eq.2) then

         do i = 1,m
            fjac(i,1) = -one
            fjac(i,2) = -one/xval(i)
            fjac(i,3) = -one/xval(i)**2
         enddo

      endif

      return
      end

c *********************************************************************
c *********************************************************************

      subroutine cxfcn2(iflag,m,n,x,fvec,fjac,ldfjac)

      implicit none

      integer m, n, ldfjac, iflag
      real*8  x(n), fvec(m), fjac(ldfjac,n)

      integer i

      integer mfit
      parameter (mfit = 30)

      real*8  omg(mfit), xval(mfit)
      common /sval/ omg, xval

      real*8  zero, one, two
      parameter (zero = 0.d0, one = 1.d0, two = 2.d0)

c ---------------------------------------------------------------------

      if (iflag.eq.1) then

         do i = 1,m
            fvec(i) = omg(i) - x(1)/(xval(i)+x(3))
     &                       - x(2)/(xval(i)+x(3))**2
         enddo

      elseif (iflag.eq.2) then

         do i = 1,m
            fjac(i,1) = -one/(xval(i)+x(3))
            fjac(i,2) = -one/(xval(i)+x(3))**2
            fjac(i,3) =     x(1)/(xval(i)+x(3))**2 +
     &                  two*x(2)/(xval(i)+x(3))**3
         enddo

      endif

      return
      end

c *********************************************************************
c *********************************************************************

      subroutine cxfcn2b(iflag,m,n,x,fvec,fjac,ldfjac)

      implicit none

      integer m, n, ldfjac, iflag
      real*8  x(n), fvec(m), fjac(ldfjac,n)

      integer i

      integer mfit
      parameter (mfit = 30)

      real*8  omg(mfit), xval(mfit)
      common /sval/ omg, xval

      real*8  zero, one, two
      parameter (zero = 0.d0, one = 1.d0, two = 2.d0)

c ---------------------------------------------------------------------

      if (iflag.eq.1) then

         do i = 1,m
            fvec(i) = omg(i) - x(2)/xval(i)
     &                       - x(3)/xval(i)**2
         enddo

      elseif (iflag.eq.2) then

         do i = 1,m
            fjac(i,1) = -one/xval(i)
            fjac(i,2) = -one/xval(i)**2
         enddo

      endif

      return
      end

c *********************************************************************
c *********************************************************************

      double precision function exp1x(x)

c ... exp(x) times
c ... Rational approximation to 1st exponential integral (within 5.e-5)
c ... (ref. Abramowitz and Stegun, p. 231

      double precision x,x0,x1,a0,a1,a2,a3,a4,a5,b1,b2,b3,b4

      data x0,x1 / 1.d-4, 1.d0 /

      data a0,a1,a2,a3,a4,a5
     &     / -.57721566d0, .99999193d0,-.24991055d0,
     &        .05519968d0,-.00976004d0, .00107857d0 /

      data b1,b2,b3,b4 / 2.334733d0,0.250621d0,3.330657d0,1.681534d0 /

c ---------------------------------------------------------------------

      if (x.lt.x0) then
         exp1x = (1.d0 + x) * (-log(x) + a0 + x*(a1 + x*a2))
      elseif (x.lt.x1) then
         exp1x = exp(x) *
     &           (-log(x) + a0 + x*(a1 + x*(a2+ x*(a3+ x*(a4 + x*a5)))))
      else
         exp1x = (b2 + x*(b1 + x)) / (b4 + x*(b3 + x)) / x
      endif

      return
      end

c *********************************************************************
c *********************************************************************

      subroutine xerbla(srname,info)

      character*6  srname
      integer      info

c ---------------------------------------------------------------------

      write (*,900) srname,info

  900 format(1x,' On entry to ', a6, ' parameter number ', i2,
     &          ' had an illegal value' )

      stop

      return
      end

c *********************************************************************
c *********************************************************************

      subroutine xsetun(iunit)

      integer  iunit

      integer  ierr, j4save

c ---------------------------------------------------------------------


      ierr = j4save(3,iunit,.true.)
      ierr = j4save(5,1,.true.)

      return
      end

c *********************************************************************
